global fid dist

dt   =    100;
tmax =  50000;

% Startposition f�r den Beginn der Simulation
x    =  2.0;
y    = -5.0;
phi  =  0.0;

% Endposition (f�r Positionsregler)
xz   =  -5.0;
yz   =  5.0;
phiz =  pi;

% Initialisierung der Robotergeschwindigkeiten
vx =  0.0;
vy =  0.0;

% Roboterradius (f�r Sonarmessungen)
radius = 0.25;

% Einschr�nkungen f�r den Geschwindigkeitsregler (Maximalwerte)
vmax      = 1.00;
phidotmax = 30*pi/180;

% Hindernisse im Raum initialisieren
koerper = default;

% clear figures
clf

% linkes Teilbild
subplot(1,2,1);
axis([-10 10 -10 10]);
set(gca, 'DataAspectRatio', [1 1 1]);
subplot(1,2,1); plotKoerper(koerper);
subplot(1,2,1); plotRobot(x, y, phi, radius, 1);
grid on;

% Daten für rechtes Teilbild
dist = sonar(x,y,phi,koerper,radius);
[ X, Y ] = pol2cart(dist(:,2)+pi/2,dist(:,1));
X = [X; X(1,:)]; Y = [Y; Y(1,:)];

% rechtes Teilbild
subplot(1,2,2);
axis([-15 15 -15 15]);
set(gca, 'DataAspectRatio', [1 1 1]);
subplot(1,2,2); hold on;

%plot(X,Y,'b-*','erase','none');
animatedline(X,Y, 'Color', 'blue', 'Marker', '*' );

%plot(0.0,0.0,'r-*','erase','none');
animatedline(0.0,0.0, 'Color', 'red', 'Marker', '*');

gif = 'behControl.gif';
f = getframe(gcf);
[im,map] = rgb2ind(f.cdata,256,'nodither');
imwrite(im,map,gif,'Delaytime',0.5,'Loopcount',inf)

fid = fopen('test.dat','w');

time = 0;
while (time < tmax),
   fprintf(fid,'t=%8.3f   x=%8.4f   y=%8.4f   phi=%8.4f   vx=%8.4f   vy=%8.4f\n',...
    time/1000, x, y, phi, vx, vy);
   fprintf(1,'t=%8.3f   x=%8.4f   y=%8.4f   phi=%8.4f   vx=%8.4f   vy=%8.4f\n',...
    time/1000, x, y, phi, vx, vy);
   
   % Repulsion
   dist = sonar(x,y,phi,koerper,radius);
   [vxrep, vyrep] = repulsion;
   
   % Freespace-Verhalten
   [vxfrs, vyfrs] = freespace;
   
   % Positionsregler
   [vxpos, vypos] = posloop(x, y, phi, xz, yz, phiz);

   % Joystick Command
   vxjoy = 0;
   vyjoy = 0;

   % ---------------------------------------------------------------
   % an dieser Stelle dann Verhaltensfusion mit Analogical Gates !!!
   % ---------------------------------------------------------------
   
   % ---------------------------------------------------------------
   %x0=invk(vxrep,vxfrs);
   %y0=invk(vyrep,vyfrs);
   
   %vx=prev(vxpos,x0);
   %vy=prev(vypos,y0);
   
   % ---------------------------------------------------------------
   %x0=invk(vxrep,vxfrs);
   %y0=invk(vyrep,vyfrs);
   
   %vx=prev(x0,vxpos);
   %vy=prev(y0,vypos);
   
   % ---------------------------------------------------------------
   x0=prev(1.3*vxfrs,vxpos);
   y0=prev(1.3*vyfrs,vypos);
   
   vx=prev(x0,.7*vxrep);
   vy=prev(y0,.7*vyrep);
   
   % ---------------------------------------------------------------
   % x0=prev(vxpos,vxjoy);
   % y0=prev(vypos,vyjoy);
   
   % x1=invk(x0,vxfrs);
   % x2=invk(y0,vyfrs);
   
   % vx=prev(x1,vxrep);
   % vy=prev(x2,vyrep);
   
   % vx = vxrep;
   % vy = vyrep;
   % vx = vxfrs;
   % vy = vyfrs;
   % vx = vxpos;
   % vy = vypos;
   % vx = vxjoy;
   % vy = vyjoy;
   % ---------------------------------------------------------------
   
   % Begrenzung der Maximalgeschwindigkeiten
   if abs(vx) > vmax
      vx = sign(vx)*vmax;
   end
   if abs(vy) > phidotmax
      vy = sign(vy)*phidotmax;
   end
      
   subplot(1,2,1);
   
   iter = iter+1;
   time = time + dt;
   
   x_old = x;
   y_old = y;
   phi_old = phi;
   [x, y, phi] = driveRobPlot(x, y, phi, vx, vy, dt/1000);
   
   % Ver�nderungen in den Fenstern anzeigen
   if ( rem(time, 10*dt) == 0 )
      subplot(1,2,1); plotRobot(x, y, phi, radius, 1);
      
      X_old = X; Y_old = Y;
      [ X, Y ] = pol2cart(dist(:,2)+pi/2,dist(:,1));
      X = [X; X(1,:)]; Y = [Y; Y(1,:)];
      subplot(1,2,2); hold on; 

      %plot(X_old,Y_old,'w-*','erase','none');
      animatedline(X_old,Y_old, 'Color', 'white', 'Marker', '*');
      
      %plot(0.0,0.0,'w-*','erase','none');
      animatedline(0.0,0.0, 'Color', 'white', 'Marker', '*');
      
      cla;
      subplot(1,2,2); hold on;

      %plot(X,Y,'b-*','erase','none');
      animatedline(X,Y, 'Color', 'blue', 'Marker', '*');

      %plot(0.0,0.0,'r-*','erase','none');
      animatedline(0.0,0.0, 'Color', 'red', 'Marker', '*');
      
      drawnow
      f = getframe(gcf);
      [im,map]=rgb2ind(f.cdata,256,'nodither');
      imwrite(im,map,gif,'writemode','append','Delaytime',0.5);
      
      subplot(1,2,1); plotRobot(x, y, phi, radius, 0);
      
      %pause;

   end
end

subplot(1,2,1); plotRobot(x, y, phi, radius, 1);

fprintf(fid,'t=%8.3f   x=%8.4f   y=%8.4f   phi=%8.4f   vx=%8.4f   vy=%8.4f\n',...
 time/1000, x, y, phi, vx, vy);
fprintf(1,'t=%8.3f   x=%8.4f   y=%8.4f   phi=%8.4f   vx=%8.4f   vy=%8.4f\n',...
 time/1000, x, y, phi, vx, vy);

fclose(fid);

